using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Camera2D
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Texture2D mario;

        Camera2D cam = new Camera2D();

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            //Fix to get rid multiple Draw() calls in a row
            //http://forums.create.msdn.com/forums/t/9934.aspx
            IsFixedTimeStep = false;
            graphics.SynchronizeWithVerticalRetrace = true;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here


            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // Ladda in mario
            mario = this.Content.Load<Texture2D>("mario");

            // Positionera vr kamera till mitten av vr tile rendering
            cam.Pos = new Vector2(mario.Width * 16 / 2, mario.Height * 9 / 2);


        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            KeyboardState keyState = Keyboard.GetState();
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || keyState.IsKeyDown(Keys.Escape))
                this.Exit();

            // Keyboard Controls
            if (keyState.IsKeyDown(Keys.Left))
                cam.pos += new Vector2(-3f, 0);
            if (keyState.IsKeyDown(Keys.Right))
                cam.pos += new Vector2(3, 0f);
            if (keyState.IsKeyDown(Keys.Up))
                cam.pos += new Vector2(0, -3f);
            if (keyState.IsKeyDown(Keys.Down))
                cam.pos += new Vector2(0, 3f);

            // Zoom
            if (keyState.IsKeyDown(Keys.PageUp))
                cam.Zoom += 0.01f;
            if (keyState.IsKeyDown(Keys.PageDown))
                cam.Zoom -= 0.01f;

            // Rot
            if (keyState.IsKeyDown(Keys.Home))
                cam.Rotation += 0.01f;
            if (keyState.IsKeyDown(Keys.End))
                cam.Rotation -= 0.01f;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            Rectangle destRect = new Rectangle(0, 0, mario.Width * 16, mario.Height * 9);

            spriteBatch.Begin(SpriteSortMode.FrontToBack,
                                BlendState.AlphaBlend,
                                SamplerState.LinearWrap,
                                DepthStencilState.Default,
                                RasterizerState.CullNone,
                                null,
                                cam.get_transformation(GraphicsDevice));

            spriteBatch.Draw(mario, Vector2.Zero, destRect, Color.White, 0, Vector2.Zero, 1, SpriteEffects.None, 0);


            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
